<?php

namespace Drupal\trash;

trait TrashStorageTrait {

  /**
   * {@inheritdoc}
   */
  public function delete(array $entities) {
    if ($this->getTrashManager()->getTrashContext() !== 'active') {
      parent::delete($entities);
      return;
    }

    $field_name = 'deleted';
    $revisionable = $this->getEntityType()->isRevisionable();

    foreach ($entities as $entity) {
      $entity->set($field_name, \Drupal::time()->getRequestTime());

      // Always create a new revision if the entity type is revisionable.
      if ($revisionable) {
        /** @var \Drupal\Core\Entity\RevisionableInterface $entity */
        $entity->setNewRevision(TRUE);
      }
      $entity->save();
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getStorageSchema() {
    if (!isset($this->storageSchema)) {
      $class = $this->entityType->getHandlerClass('storage_schema') ?: 'Drupal\Core\Entity\Sql\SqlContentEntityStorageSchema';

      // Ensure that we use our generated storage schema class.
      $class = _trash_generate_storage_class($class, 'storage_schema');

      $this->storageSchema = new $class($this->entityTypeManager, $this->entityType, $this, $this->database, $this->entityFieldManager);
    }
    return $this->storageSchema;
  }

  /**
   * Gets the trash manager service.
   */
  private function getTrashManager(): TrashManagerInterface {
    return \Drupal::service('trash.manager');
  }

}
